@@ -24,7 +24,7 @@ from presto.Defaults import default_out_args
 from presto.IO import  printLog, printMessage, printWarning, printError, printDebug
 from changeo.Defaults import default_format
 from changeo.IO import splitName, getDbFields, getFormatOperators, getOutputHandle, getOutputName
-from changeo.Alignment import getRegions
+from changeo.Alignment import RegionDefinition
 from changeo.Commandline import CommonHelpFormatter, checkArgs, getCommonArgParser, parseCommonArgs
 
 
@@ -219,17 +219,21 @@ def maskSplitCodons(receptor, recursive=False, mask=True):
 
     #tally where --- gaps are in IMGT sequence and remove them for now
     gaps = []
+    ndotgaps = []
     nsi = ""
     for i in range(0,len(si)):
         if si[i] == "-":
             gaps.append(1)
+            ndotgaps.append(1)
         else:
             gaps.append(0)
             nsi = nsi + si[i]
+            if si[i] != ".":
+                ndotgaps.append(0)
 
     #find any gaps not divisible by three
     curgap = 0
-    for i in gaps:
+    for i in ndotgaps:
         if i == 1:
             curgap += 1
         elif i == 0 and curgap != 0:
@@ -481,6 +485,9 @@ def characterizePartitionErrors(sequences, clones, meta_data):
     nseqs = len(sequences)
     imgtar = clones[0].getField("imgtpartlabels")
     germline = clones[0].getField("germline_imgt_d_mask")
+    if germline is "":
+        germline = clones[0].getField("germline_imgt")
+
     correctseqs = False
 
     for seqi in range(0, len(sequences)):
@@ -532,7 +539,10 @@ def characterizePartitionErrors(sequences, clones, meta_data):
     #Resolve germline if there are differences, e.g. if reconstruction was done before clonal clustering
     resolveglines = False
     for c in clones:
-        if c.getField("germline_imgt_d_mask") != germline:
+        ngermline = c.getField("germline_imgt_d_mask")
+        if ngermline is "":
+            ngermline = c.getField("germline_imgt")
+        if ngermline != germline:
             resolveglines = True
 
     if resolveglines:
@@ -746,7 +756,7 @@ def outputIgPhyML(clones, sequences, meta_data=None, collapse=False, ncdr3=False
         return nseqs
 
 
-def maskCodonsLoop(r, clones, cloneseqs, logs, fails, out_args, fail_writer):
+def maskCodonsLoop(r, clones, cloneseqs, logs, fails, out_args, fail_writer, mask=True):
     """
     Masks codons split by alignment to IMGT reference
 
@@ -768,7 +778,6 @@ def maskCodonsLoop(r, clones, cloneseqs, logs, fails, out_args, fail_writer):
     if r.dupcount is None:
         r.dupcount = 1
     fails["rec_count"] += 1
-    fails["totalreads"] += 1
     #printProgress(rec_count, rec_count, 0.05, start_time)
     ptcs = hasPTC(r.sequence_imgt)
     gptcs = hasPTC(r.getField("germline_imgt_d_mask"))
@@ -787,8 +796,8 @@ def maskCodonsLoop(r, clones, cloneseqs, logs, fails, out_args, fail_writer):
 
     if r.functional and ptcs < 0:
         #If IMGT regions are provided, record their positions
-        regions = getRegions(r.sequence_imgt, r.junction_length)
-        #print(regions["cdr1_imgt"]+regions["fwr4_imgt"])
+        rd = RegionDefinition(r.junction_length, amino_acid=False)
+        regions = rd.getRegions(r.sequence_imgt)
         if regions["cdr3_imgt"] is not "" and regions["cdr3_imgt"] is not None:
             simgt = regions["fwr1_imgt"] + regions["cdr1_imgt"] + regions["fwr2_imgt"] + regions["cdr2_imgt"] + \
                     regions["fwr3_imgt"] + regions["cdr3_imgt"] + regions["fwr4_imgt"]
@@ -814,11 +823,38 @@ def maskCodonsLoop(r, clones, cloneseqs, logs, fails, out_args, fail_writer):
                 fails["seq_fail"] += 1
                 fails["region_fail"] += 1
                 return 0
+
+        elif regions["fwr3_imgt"] is not "" and regions["fwr3_imgt"] is not None:
+            simgt = regions["fwr1_imgt"] + regions["cdr1_imgt"] + regions["fwr2_imgt"] + regions["cdr2_imgt"] + \
+                    regions["fwr3_imgt"]
+            nseq = r.sequence_imgt[len(simgt):len(r.sequence_imgt)]
+
+            if len(simgt) < len(r.sequence_imgt):
+                simgt = regions["fwr1_imgt"] + regions["cdr1_imgt"] + regions["fwr2_imgt"] + \
+                        regions["cdr2_imgt"] + regions["fwr3_imgt"] + nseq
+            imgtpartlabels = [13] * len(regions["fwr1_imgt"]) + [30] * len(regions["cdr1_imgt"]) + [45] * len(
+                regions["fwr2_imgt"]) + \
+                             [60] * len(regions["cdr2_imgt"]) + [80] * len(regions["fwr3_imgt"]) + \
+                             [108] * int(len(nseq))
+            r.setField("imgtpartlabels", imgtpartlabels)
+            if len(r.getField("imgtpartlabels")) != len(r.sequence_imgt) or simgt != r.sequence_imgt:
+                log = OrderedDict()
+                log["ID"] = r.sequence_id
+                log["CLONE"] = r.clone
+                log["SEQ_IN"] = r.sequence_input
+                log["SEQ_IMGT"] = r.sequence_imgt
+                logs[r.sequence_id] = log
+                logs[r.sequence_id]["PASS"] = False
+                logs[r.sequence_id]["FAIL"] = "FWR/CDR error"
+                logs[r.sequence_id]["FWRCDRSEQ"] = simgt
+                fails["seq_fail"] += 1
+                fails["region_fail"] += 1
+                return 0
         else:
             #imgt_warn = "\n! IMGT FWR/CDR sequence columns not detected.\n! Cannot run CDR/FWR partitioned model on this data.\n"
             imgtpartlabels = [0] * len(r.sequence_imgt)
             r.setField("imgtpartlabels", imgtpartlabels)
-        mout = maskSplitCodons(r)
+        mout = maskSplitCodons(r, mask=mask)
         mask_seq = mout[0]
         ptcs = hasPTC(mask_seq)
         if ptcs >= 0:
@@ -863,7 +899,7 @@ def maskCodonsLoop(r, clones, cloneseqs, logs, fails, out_args, fail_writer):
 
 # Run IgPhyML on outputed data
 def runIgPhyML(outfile, igphyml_out, clone_dir, nproc=1, optimization="lr", omega="e,e", kappa="e", motifs="FCH",
-               hotness="e,e,e,e,e,e",oformat="tab", nohlp=False, clean="none"):
+               hotness="e,e,e,e,e,e", oformat="tab", nohlp=False, asr=-1, clean="none"):
     """
     Run IgPhyML on outputted data
 
@@ -892,6 +928,10 @@ def runIgPhyML(outfile, igphyml_out, clone_dir, nproc=1, optimization="lr", omeg
                 optimization, "--omega", omega, "-t", kappa, "--motifs", motifs, "--hotness", hotness, "--oformat",
                 oformat, "--outname", igphyml_out]
 
+    if asr >= 0:
+        hlp_args.append("--ASRc")
+        hlp_args.append(str(asr))
+
     log = OrderedDict()
     log["START"] = "IgPhyML GY94 tree estimation"
     printLog(log)
@@ -967,10 +1007,10 @@ def runIgPhyML(outfile, igphyml_out, clone_dir, nproc=1, optimization="lr", omeg
 
 
 # Note: Collapse can give misleading dupcount information if some sequences have ambiguous characters at polymorphic sites
-def buildTrees(db_file, meta_data=None, target_clones=None, collapse=False, ncdr3=False, sample_depth=-1, min_seq=1,append=None,
-               igphyml=False, nproc=1, optimization="lr", omega="e,e", kappa="e", motifs="FCH",
-               hotness="e,e,e,e,e,e", oformat="tab", clean="none", nohlp=False,
-               format=default_format, out_args=default_out_args):
+def buildTrees(db_file, meta_data=None, target_clones=None, collapse=False, ncdr3=False, nmask=False,
+               sample_depth=-1, min_seq=1,append=None, igphyml=False, nproc=1, optimization="lr", omega="e,e",
+               kappa="e", motifs="FCH", hotness="e,e,e,e,e,e", oformat="tab", clean="none", nohlp=False,
+               asr=-1, format=default_format, out_args=default_out_args):
     """
     Masks codons split by alignment to IMGT reference, then produces input files for IgPhyML
 
@@ -980,6 +1020,7 @@ def buildTrees(db_file, meta_data=None, target_clones=None, collapse=False, ncdr
       target_clones (str): List of clone IDs to analyze.
       collapse (bool): if True collapse identical sequences.
       ncdr3 (bool): if True remove all CDR3s.
+      nmask (bool): if False, do not attempt to mask split codons
       sample_depth (int): depth of subsampling before deduplication
       min_seq (int): minimum number of sequences per clone
       append (str): column name to append to sequence_id
@@ -1088,6 +1129,9 @@ def buildTrees(db_file, meta_data=None, target_clones=None, collapse=False, ncdr
             if init_clone_sizes[r.clone] >= min_seq:
                big_enough.append(r)
 
+    fails["totalreads"] = len(all_records)
+    #fails["minseq_fail"] =  len(all_records) - len(big_enough)         
+
     if len(big_enough) == 0:
         printError("\n\nNo sequences found that match specified criteria.",1)
 
@@ -1104,11 +1148,25 @@ def buildTrees(db_file, meta_data=None, target_clones=None, collapse=False, ncdr
 
         r.sequence_id = r.sequence_id.replace(",","-") #remove commas from sequence ID
         r.sequence_id = r.sequence_id.replace(":","-") #remove colons from sequence ID
+        r.sequence_id = r.sequence_id.replace(",","-") #remove commas from sequence ID
+        r.sequence_id = r.sequence_id.replace(")","-") #remove parenthesis from sequence ID
+        r.sequence_id = r.sequence_id.replace("(","-") #remove parenthesis from sequence ID
+        if(meta_data is not None):
+            for m in range(0,len(meta_data)):
+                md = r.getField(meta_data[m])
+                md = md.replace(",","-") #remove commas from metadata
+                md = md.replace(":","-") #remove colons from metadata
+                md = md.replace(",","-") #remove commas from metadata
+                md = md.replace(")","-") #remove parenthesis from metadata
+                md = md.replace("(","-") #remove parenthesis from metadata
+                r.setField(meta_data[m],md)
+
         if append is not None:
             if append is not None:
                 for m in append:
                     r.sequence_id = r.sequence_id + "_" + r.getField(m)
-        total += maskCodonsLoop(r, clones, cloneseqs, logs, fails, out_args, fail_writer)
+        total += maskCodonsLoop(r, clones, cloneseqs, logs, fails, out_args, fail_writer,
+                                mask = not nmask)
         if total == sample_depth:
             break
 
@@ -1171,7 +1229,8 @@ def buildTrees(db_file, meta_data=None, target_clones=None, collapse=False, ncdr
     #printProgress(rec_count, rec_count, 0.05, start_time)
     log = OrderedDict()
     log["OUTPUT"] = os.path.basename(pass_handle.name) if pass_handle is not None else None
-    log["RECORDS"] = fails["rec_count"]
+    log["RECORDS"] = fails["totalreads"]
+    log["INITIAL_FILTER"] = fails["rec_count"]
     log["PASS"] = pass_count
     log["FAIL"] = fail_count
     log["NONFUNCTIONAL"] = fails["nf_fail"]
@@ -1191,7 +1250,7 @@ def buildTrees(db_file, meta_data=None, target_clones=None, collapse=False, ncdr
     if igphyml:
         runIgPhyML(pass_handle.name, igphyml_out=igphyml_out, clone_dir=clone_dir, nproc=nproc,
                    optimization=optimization, omega=omega, kappa=kappa, motifs=motifs,
-                   hotness=hotness, oformat=oformat, nohlp=nohlp,clean=clean)
+                   hotness=hotness, oformat=oformat, nohlp=nohlp,clean=clean,asr=asr)
 
     return output
 
@@ -1217,9 +1276,9 @@ def getArgParser():
                     parameter estimates and lineage trees from running IgPhyML, if specified
 
              required fields:
-                 SEQUENCE_ID, SEQUENCE_INPUT, SEQUENCE_IMGT,
-                 GERMLINE_IMGT_D_MASK, V_CALL, J_CALL, CLONE,
-                 V_SEQ_START
+                 sequence_id, sequence, sequence_alignment,
+                 germline_alignment_d_mask or germline_alignment,
+                 v_call, j_call, clone_id, v_sequence_start
               """)
 
     # Parent parser
@@ -1235,6 +1294,8 @@ def getArgParser():
                         help="""If specified, collapse identical sequences before exporting to fasta.""")
     group.add_argument("--ncdr3", action="store_true", dest="ncdr3",
                         help="""If specified, remove CDR3 from all sequences.""")
+    group.add_argument("--nmask", action="store_true", dest="nmask",
+                       help="""If specified, do not attempt to mask split codons.""")
     group.add_argument("--md", nargs="+", action="store", dest="meta_data",
                        help="""List of fields to containing metadata to include in output fasta file 
                             sequence headers.""")
@@ -1280,6 +1341,8 @@ def getArgParser():
                                help="""IgPhyML output format.""")
     igphyml_group.add_argument("--nohlp", action="store_true", dest="nohlp",
                                help="""Don't run HLP model?""")
+    igphyml_group.add_argument("--asr", action="store", dest="asr", type=float, default=-1,
+                               help="""Ancestral sequence reconstruction interval (0-1).""")
 
     return parser
 
