@@ -30,7 +30,7 @@ from changeo.Applications import default_tbl2asn_exec, runASN
 from changeo.Defaults import default_id_field, default_seq_field, default_germ_field, \
                              default_csv_size, default_format, default_out_args
 from changeo.Commandline import CommonHelpFormatter, checkArgs, getCommonArgParser, parseCommonArgs
-from changeo.Gene import c_gene_regex, parseAllele, buildGermline
+from changeo.Gene import getCGene, buildGermline
 from changeo.IO import countDbFile, getFormatOperators, getOutputHandle, AIRRReader, AIRRWriter, \
                        ChangeoReader, ChangeoWriter, TSVReader, ReceptorData, readGermlines, \
                        checkFields, yamlDict
@@ -254,8 +254,12 @@ def convertToAIRR(db_file, format=default_format,
 
     # Set output fields replacing length with end fields
     in_fields = [schema.toReceptor(f) for f in db_iter.fields]
-    out_fields = [ReceptorData.length_fields[f][1] if f in ReceptorData.length_fields else f \
-                  for f in in_fields]
+    out_fields = []
+    for f in in_fields:
+        if f in ReceptorData.length_fields and ReceptorData.length_fields[f][0] in in_fields:
+            out_fields.append(ReceptorData.length_fields[f][1])
+        out_fields.append(f)
+    out_fields = list(OrderedDict.fromkeys(out_fields))
     out_fields = [AIRRSchema.fromReceptor(f) for f in out_fields]
 
     # Open output writer
@@ -318,8 +322,12 @@ def convertToChangeo(db_file, out_file=None, out_args=default_out_args):
 
     # Set output fields replacing length with end fields
     in_fields = [AIRRSchema.toReceptor(f) for f in db_iter.fields]
-    out_fields = [ReceptorData.end_fields[f][1] if f in ReceptorData.end_fields else f \
-                  for f in in_fields]
+    out_fields = []
+    for f in in_fields:
+        out_fields.append(f)
+        if f in ReceptorData.end_fields and ReceptorData.end_fields[f][0] in in_fields:
+            out_fields.append(ReceptorData.end_fields[f][1])
+    out_fields = list(OrderedDict.fromkeys(out_fields))
     out_fields = [ChangeoSchema.fromReceptor(f) for f in out_fields]
 
     # Open output writer
@@ -542,8 +550,8 @@ def makeGenbankFeatures(record, start=None, end=None, product=default_product,
 
     Arguments:
       record : Receptor record.
-      start : start position of the modified seqeuence in the input sequence. Used for feature position offsets.
-      end : end position of the modified seqeuence in the input sequence. Used for feature position offsets.
+      start : start position of the modified sequence in the input sequence. Used for feature position offsets.
+      end : end position of the modified sequence in the input sequence. Used for feature position offsets.
       product : Product (protein) name.
       inference : Reference alignment tool.
       db_xref : Reference database name.
@@ -578,7 +586,7 @@ def makeGenbankFeatures(record, start=None, end=None, product=default_product,
         j_allele = record.getJAlleleNumber()
         # C region
         if c_field is not None:
-            c_gene = parseAllele(record.getField(c_field), c_gene_regex, action='first')
+            c_gene = getCGene(record.getField(c_field), action='first')
     else:
         # V gene
         v_split = iter(record.v_call.rsplit(allele_delim, maxsplit=1))
@@ -695,6 +703,8 @@ def makeGenbankFeatures(record, start=None, end=None, product=default_product,
     #   codon_start (must indicate codon offset)
     #   function = JUNCTION
     #   inference
+    # print(record.v_germ_end_imgt, record.v_seq_end, record.v_germ_end_imgt - 310)
+    # print(record.junction_start, record.junction_end, record.junction_length)
     if record.junction_start is not None and record.junction_end is not None:
         # Define junction boundaries
         junction_start = record.junction_start - start_trim
@@ -944,13 +954,12 @@ def getArgParser():
                      feature tables and fasta files containing MiAIRR compliant input for tbl2asn.
 
              required fields:
-                 SEQUENCE_ID, SEQUENCE_INPUT, JUNCTION, V_CALL, D_CALL, J_CALL, 
-                 V_SEQ_START, V_SEQ_LENGTH, D_SEQ_START, D_SEQ_LENGTH, J_SEQ_START, J_SEQ_LENGTH,
-                 NP1_LENGTH, NP2_LENGTH
-                 SEQUENCE_IMGT, V_GERM_START_IMGT, V_GERM_LENGTH_IMGT
+                 sequence_id, sequence, sequence_alignment, junction, v_call, d_call, j_call, 
+                 v_germline_start, v_germline_end, v_sequence_start, v_sequence_end, 
+                 d_sequence_start, d_sequence_end, j_sequence_start, j_sequence_end 
                  
              optional fields:
-                 GERMLINE_IMGT, GERMLINE_IMGT_D_MASK, CLONE, C_CALL
+                 germline_alignment, c_call, clone_id 
              ''')
     
     # Define ArgumentParser
