@@ -19,8 +19,7 @@ from time import time
 
 # Presto and changeo imports
 from presto.IO import printLog, printProgress, printMessage
-from changeo.Defaults import default_id_field, default_seq_field, default_germ_field, \
-                             default_csv_size, default_out_args
+from changeo.Defaults import default_csv_size, default_out_args
 from changeo.Commandline import CommonHelpFormatter, checkArgs, getCommonArgParser, parseCommonArgs
 from changeo.IO import countDbFile, getOutputHandle, splitName, TSVReader, TSVWriter
 
@@ -845,12 +844,7 @@ def getArgParser():
                      the subcommands add, index, drop, delete, rename, select, sort or update.
 
              required fields:
-                 SEQUENCE_ID
-                 
-             optional fields:
-                 JUNCTION, SEQUENCE_IMGT, SEQUENCE_VDJ, GERMLINE_IMGT, GERMLINE_VDJ,
-                 GERMLINE_IMGT_D_MASK, GERMLINE_VDJ_D_MASK,
-                 GERMLINE_IMGT_V_REGION, GERMLINE_VDJ_V_REGION
+                 sequence_id
              ''')
     
     # Define ArgumentParser
