@@ -20,10 +20,10 @@ from Bio.Seq import Seq
 # Presto and changeo imports
 from presto.IO import getFileType, printError, printWarning
 from changeo.Defaults import default_csv_size
-from changeo.Gene import allele_regex, v_allele_regex, d_allele_regex, j_allele_regex, locus_regex, \
-                         parseAllele
-from changeo.Receptor import AIRRSchema, ChangeoSchema, Receptor, ReceptorData
-from changeo.Alignment import decodeBTOP, encodeCIGAR, padAlignment, gapV, inferJunction, getRegions
+from changeo.Gene import getAllele, getLocus, getVAllele, getDAllele, getJAllele
+from changeo.Receptor import AIRRSchema, AIRRSchemaAA, ChangeoSchema, ChangeoSchemaAA, Receptor, ReceptorData
+from changeo.Alignment import decodeBTOP, encodeCIGAR, padAlignment, gapV, inferJunction, \
+                              RegionDefinition, getRegions
 
 # System settings
 csv.field_size_limit(default_csv_size)
@@ -185,7 +185,7 @@ class ChangeoWriter(TSVWriter):
     """
     Writes Change-O formatted data.
     """
-    def __init__(self, handle, fields=ChangeoSchema.standard_fields, header=True):
+    def __init__(self, handle, fields=ChangeoSchema.required, header=True):
         """
         Initializer
 
@@ -304,7 +304,7 @@ class AIRRWriter(TSVWriter):
     """
     Writes AIRR formatted data.
     """
-    def __init__(self, handle, fields=AIRRSchema.standard_fields):
+    def __init__(self, handle, fields=AIRRSchema.required):
         """
         Initializer
 
@@ -538,8 +538,8 @@ class IMGTReader:
         j_call = delim_regex.sub(',', clean_regex.sub('', j_str)) if j_str else None
 
         # Locus
-        locus_list = [parseAllele(v_call, locus_regex, action='first'),
-                      parseAllele(j_call, locus_regex, action='first')]
+        locus_list = [getLocus(v_call, action='first'),
+                      getLocus(j_call, action='first')]
         locus = set(filter(None, locus_list))
 
         # Result
@@ -803,7 +803,9 @@ class IMGTReader:
 
         # Parse optional fields
         db.update(IMGTReader._parseScores(summary))
-        db.update(getRegions(db['sequence_imgt'], db['junction_length']))
+        rd = RegionDefinition(junction_length=db.get('junction_length', None),
+                              amino_acid=False, definition='default')
+        db.update(rd.getRegions(db.get('sequence_imgt', None)))
         db.update(IMGTReader._parseJuncDetails(junction))
 
         return db
@@ -844,14 +846,11 @@ class IgBLASTReader:
     """
     # Ordered list of known fields
     @staticmethod
-    def customFields(scores=False, regions=False, cdr3=False, schema=None):
+    def customFields(schema=None):
         """
         Returns non-standard fields defined by the parser
 
         Arguments:
-          scores : if True include alignment scoring fields.
-          regions : if True include IMGT-gapped CDR and FWR region fields.
-          cdr3 : if True include IgBLAST CDR3 assignment fields.
           schema : schema class to pass field through for conversion.
                    If None, return changeo.Receptor.Receptor attribute names.
 
@@ -859,36 +858,25 @@ class IgBLASTReader:
           list : list of field names.
         """
         # IgBLAST scoring fields
-        score_fields = ['v_score',
-                        'v_identity',
-                        'v_evalue',
-                        'v_cigar',
-                        'd_score',
-                        'd_identity',
-                        'd_evalue',
-                        'd_cigar',
-                        'j_score',
-                        'j_identity',
-                        'j_evalue',
-                        'j_cigar']
-
-        # FWR amd CDR fields
-        region_fields = ['fwr1_imgt',
-                         'fwr2_imgt',
-                         'fwr3_imgt',
-                         'fwr4_imgt',
-                         'cdr1_imgt',
-                         'cdr2_imgt',
-                         'cdr3_imgt']
-
-        # IgBLAST CDR3 fields
-        cdr3_fields = ['cdr3_igblast',
-                       'cdr3_igblast_aa']
-        
-        fields = []
-        if scores:  fields.extend(score_fields)
-        if regions:  fields.extend(region_fields)
-        if cdr3:  fields.extend(cdr3_fields)
+        fields = ['v_score',
+                  'v_identity',
+                  'v_evalue',
+                  'v_cigar',
+                  'd_score',
+                  'd_identity',
+                  'd_evalue',
+                  'd_cigar',
+                  'j_score',
+                  'j_identity',
+                  'j_evalue',
+                  'j_cigar',
+                  'fwr1_imgt',
+                  'fwr2_imgt',
+                  'fwr3_imgt',
+                  'fwr4_imgt',
+                  'cdr1_imgt',
+                  'cdr2_imgt',
+                  'cdr3_imgt']
 
         # Convert field names if schema provided
         if schema is not None:
@@ -896,7 +884,7 @@ class IgBLASTReader:
 
         return fields
 
-    def __init__(self, igblast, sequences, references, asis_calls=False, receptor=True):
+    def __init__(self, igblast, sequences, references, asis_calls=False, regions='default', receptor=True):
         """
         Initializer.
 
@@ -906,6 +894,7 @@ class IgBLASTReader:
                             sequence descriptions as keys with original query sequences as SeqRecord values.
           references (dict): dictionary of IMGT gapped germline sequences.
           asis_calls (bool): if True do not parse gene calls for allele names.
+          regions (str): name of the IMGT FWR/CDR region definitions to use.
           receptor (bool): if True (default) iteration returns an Receptor object, otherwise it returns a dictionary.
 
         Returns:
@@ -915,14 +904,14 @@ class IgBLASTReader:
         self.igblast = igblast
         self.sequences = sequences
         self.references = references
+        self.regions = regions
         self.asis_calls = asis_calls
         self.receptor = receptor
 
         # Define parsing blocks
-        self.groups = groupby(self.igblast, lambda x: not re.match('# IGBLASTN', x))
+        self.groups = groupby(self.igblast, lambda x: not re.match('# IGBLAST', x))
 
-    @staticmethod
-    def _parseQueryChunk(chunk):
+    def _parseQueryChunk(self, chunk):
         """
         Parse query section
 
@@ -937,8 +926,7 @@ class IgBLASTReader:
 
         return query.replace('# Query: ', '', 1)
 
-    @staticmethod
-    def _parseSummaryChunk(chunk):
+    def _parseSummaryChunk(self, chunk):
         """
         Parse summary section
 
@@ -972,8 +960,7 @@ class IgBLASTReader:
 
         return summary
 
-    @staticmethod
-    def _parseSubregionChunk(chunk):
+    def _parseSubregionChunk(self, chunk):
         """
         Parse CDR3 sequences generated by IgBLAST
 
@@ -1011,8 +998,7 @@ class IgBLASTReader:
 
         return cdr
 
-    @staticmethod
-    def _parseHitsChunk(chunk):
+    def _parseHitsChunk(self, chunk):
         """
         Parse hits section
 
@@ -1033,9 +1019,7 @@ class IgBLASTReader:
 
         return hits
 
-    # Parse summary results
-    @staticmethod
-    def _parseSummarySection(summary, db, asis_calls=False):
+    def _parseSummarySection(self, summary, db, asis_calls=False):
         """
         Parse summary section
 
@@ -1050,9 +1034,9 @@ class IgBLASTReader:
         result = {}
         # Parse V, D, and J calls
         if not asis_calls:
-            v_call = parseAllele(summary['v_match'], v_allele_regex, action='list')
-            d_call = parseAllele(summary['d_match'], d_allele_regex, action='list')
-            j_call = parseAllele(summary['j_match'], j_allele_regex, action='list')
+            v_call = getVAllele(summary['v_match'], action='list')
+            d_call = getDAllele(summary['d_match'], action='list')
+            j_call = getJAllele(summary['j_match'], action='list')
             result['v_call'] = ','.join(v_call) if v_call else None
             result['d_call'] = ','.join(d_call) if d_call else None
             result['j_call'] = ','.join(j_call) if j_call else None
@@ -1082,8 +1066,7 @@ class IgBLASTReader:
 
         return result
 
-    @staticmethod
-    def _parseSubregionSection(section, sequence):
+    def _parseSubregionSection(self, section, sequence):
         """
         Parse subregion section
 
@@ -1112,8 +1095,7 @@ class IgBLASTReader:
                 'junction_start': junc_start,
                 'junction_end': junc_end}
 
-    @staticmethod
-    def _parseVHitPos(v_hit):
+    def _parseVHitPos(self, v_hit):
         """
         Parse V alignment positions
 
@@ -1134,8 +1116,7 @@ class IgBLASTReader:
 
         return result
 
-    @staticmethod
-    def _parseDHitPos(d_hit, overlap):
+    def _parseDHitPos(self, d_hit, overlap):
         """
         Parse D alignment positions
 
@@ -1156,8 +1137,7 @@ class IgBLASTReader:
 
         return result
 
-    @staticmethod
-    def _parseJHitPos(j_hit, overlap):
+    def _parseJHitPos(self, j_hit, overlap):
         """
         Parse J alignment positions
 
@@ -1176,8 +1156,7 @@ class IgBLASTReader:
 
         return result
 
-    @staticmethod
-    def _appendSeq(seq, hits, start, trim=True):
+    def _appendSeq(self, seq, hits, start, trim=True):
         """
         Append aligned query sequence segment
 
@@ -1205,8 +1184,7 @@ class IgBLASTReader:
 
         return seq
 
-    @staticmethod
-    def _parseVHits(hits, db):
+    def _parseVHits(self, hits, db):
         """
         Parse V hit sub-table
 
@@ -1223,15 +1201,14 @@ class IgBLASTReader:
         v_hit = next(x for x in hits if x['segment'] == 'V')
 
         # Alignment positions
-        result.update(IgBLASTReader._parseVHitPos(v_hit))
+        result.update(self._parseVHitPos(v_hit))
         # Update VDJ sequence with and without removing insertions
-        result['sequence_vdj'] = IgBLASTReader._appendSeq(seq_vdj, v_hit, 0, trim=False)
-        result['sequence_trim'] = IgBLASTReader._appendSeq(seq_trim, v_hit, 0, trim=True)
+        result['sequence_vdj'] = self._appendSeq(seq_vdj, v_hit, 0, trim=False)
+        result['sequence_trim'] = self._appendSeq(seq_trim, v_hit, 0, trim=True)
 
         return result
 
-    @staticmethod
-    def _parseDHits(hits, db):
+    def _parseDHits(self, hits, db):
         """
         Parse D hit sub-table
 
@@ -1264,15 +1241,15 @@ class IgBLASTReader:
                     seq_trim += db['sequence_input'][np1_start:np1_end]
 
         # D alignment positions
-        result.update(IgBLASTReader._parseDHitPos(d_hit, overlap))
+        result.update(self._parseDHitPos(d_hit, overlap))
         # Update VDJ sequence with and without removing insertions
-        result['sequence_vdj'] = IgBLASTReader._appendSeq(seq_vdj, d_hit, overlap, trim=False)
-        result['sequence_trim'] = IgBLASTReader._appendSeq(seq_trim, d_hit, overlap, trim=True)
+        result['sequence_vdj'] = self._appendSeq(seq_vdj, d_hit, overlap, trim=False)
+        result['sequence_trim'] = self._appendSeq(seq_trim, d_hit, overlap, trim=True)
 
         return result
 
-    @staticmethod
-    def _parseJHits(hits, db):
+
+    def _parseJHits(self, hits, db):
         """
         Parse J hit sub-table
 
@@ -1320,15 +1297,14 @@ class IgBLASTReader:
             result['np1_length'] = 0
 
         # J alignment positions
-        result.update(IgBLASTReader._parseJHitPos(j_hit, overlap))
+        result.update(self._parseJHitPos(j_hit, overlap))
         # Update VDJ sequence with and without removing insertions
-        result['sequence_vdj'] = IgBLASTReader._appendSeq(seq_vdj, j_hit, overlap, trim=False)
-        result['sequence_trim'] = IgBLASTReader._appendSeq(seq_trim, j_hit, overlap, trim=True)
+        result['sequence_vdj'] = self._appendSeq(seq_vdj, j_hit, overlap, trim=False)
+        result['sequence_trim'] = self._appendSeq(seq_trim, j_hit, overlap, trim=True)
 
         return result
 
-    @staticmethod
-    def _parseHitScores(hits, segment):
+    def _parseHitScores(self, hits, segment):
         """
         Parse alignment scores
 
@@ -1393,7 +1369,7 @@ class IgBLASTReader:
         #     'V-(D)-J rearrangement summary': (Top V gene match, Top D gene match, Top J gene match, Chain type, stop codon, V-J frame, Productive, Strand)
         #     'V-(D)-J junction details': (V end, V-D junction, D region, D-J junction, J start)
         #     'Alignment summary': (from, to, length, matches, mismatches, gaps, percent identity)
-        #     'subregion': (nucleotide sequence, translation)
+        #     'subregion': (nucleotide sequence, translation, start, end)
         #
         #   Ignored sections
         #     'junction': '# V-(D)-J junction details'
@@ -1498,7 +1474,8 @@ class IgBLASTReader:
                                       j_germ_length=db['j_germ_length'],
                                       j_call=db['j_call'],
                                       references=self.references,
-                                      asis_calls=self.asis_calls)
+                                      asis_calls=self.asis_calls,
+                                      regions=self.regions)
             db.update(junc_dict)
 
         # Add IgBLAST CDR3 sequences
@@ -1510,7 +1487,9 @@ class IgBLASTReader:
             db.update({'cdr3_igblast': None, 'cdr3_igblast_aa': None})
 
         # Add FWR and CDR regions
-        db.update(getRegions(db.get('sequence_imgt', None), db.get('junction_length', None)))
+        rd = RegionDefinition(junction_length=db.get('junction_length', None),
+                              amino_acid=False, definition=self.regions)
+        db.update(rd.getRegions(db.get('sequence_imgt', None)))
 
         return db
 
@@ -1549,6 +1528,149 @@ class IgBLASTReader:
             return db
 
 
+class IgBLASTReaderAA(IgBLASTReader):
+    """
+    An iterator to read and parse IgBLAST amino acid alignment output files
+    """
+    @staticmethod
+    def customFields(schema=None):
+        """
+        Returns non-standard fields defined by the parser
+
+        Arguments:
+          schema : schema class to pass field through for conversion.
+                   If None, return changeo.Receptor.Receptor attribute names.
+
+        Returns:
+          list : list of field names.
+        """
+        # IgBLAST scoring fields
+        fields = ['v_score',
+                  'v_identity',
+                  'v_evalue',
+                  'v_cigar',
+                  'fwr1_aa_imgt',
+                  'fwr2_aa_imgt',
+                  'fwr3_aa_imgt',
+                  'cdr1_aa_imgt',
+                  'cdr2_aa_imgt']
+
+        # Convert field names if schema provided
+        if schema is not None:
+            fields = [schema.fromReceptor(f) for f in fields]
+
+        return fields
+
+    def _parseVHitPos(self, v_hit):
+        """
+        Parse V alignment positions
+
+        Arguments:
+          v_hit :  V alignment row from the hit table
+
+        Returns:
+          dict: db of D starts and lengths
+        """
+        result = {}
+        # Germline positions
+        result['v_germ_aa_start_vdj'] = int(v_hit['s. start'])
+        result['v_germ_aa_length_vdj'] = int(v_hit['s. end']) - result['v_germ_aa_start_vdj'] + 1
+        # Query sequence positions
+        result['v_seq_aa_start'] = int(v_hit['q. start'])
+        result['v_seq_aa_length'] = int(v_hit['q. end']) - result['v_seq_aa_start'] + 1
+        result['indels'] = 'F' if int(v_hit['gap opens']) == 0 else 'T'
+
+        return result
+
+    def _parseVHits(self, hits, db):
+        """
+        Parse V hit sub-table
+
+        Arguments:
+          hits :  hit table as a list of dictionaries.
+          db : database dictionary containing summary results.
+
+        Returns:
+          dict : db of results.
+        """
+        result = {}
+        seq_vdj = db['sequence_aa_vdj']
+        seq_trim = db['sequence_aa_trim']
+        v_hit = next(x for x in hits if x['segment'] == 'V')
+
+        # Alignment positions
+        result.update(self._parseVHitPos(v_hit))
+
+        # Assign V gene and update VDJ sequence with and without removing insertions
+        result['v_call'] = v_hit['subject id']
+        result['sequence_aa_vdj'] = self._appendSeq(seq_vdj, v_hit, 0, trim=False)
+        result['sequence_aa_trim'] = self._appendSeq(seq_trim, v_hit, 0, trim=True)
+
+        # Derived functionality
+        result['locus'] = getLocus(result['v_call'], action='first')
+        result['stop'] = '*' in result['sequence_aa_vdj']
+
+        return result
+
+    def parseSections(self, sections):
+        """
+        Parses an IgBLAST sections into a db dictionary
+
+        Arguments:
+            sections : dictionary of parsed sections from parseBlock.
+
+        Returns:
+          dict : db entries.
+        """
+        # Initialize dictionary with input sequence and id
+        db = {}
+        if 'query' in sections:
+            query = sections['query']
+            db['sequence_id'] = query
+            db['sequence_aa_input'] = str(self.sequences[query].seq)
+
+        # Parse hit table
+        if 'hits' in sections:
+            db['v_call'] = ''
+            db['sequence_aa_vdj'] = ''
+            db['sequence_aa_trim'] = ''
+            db.update(self._parseVHits(sections['hits'], db))
+            db.update(self._parseHitScores(sections['hits'], 'v'))
+
+        # Create IMGT-gapped sequence
+        if ('v_call' in db and db['v_call']) and ('sequence_aa_trim' in db and db['sequence_aa_trim']):
+            try:
+                gap = gapV(db['sequence_aa_trim'],
+                           v_germ_start=db['v_germ_aa_start_vdj'],
+                           v_germ_length=db['v_germ_aa_length_vdj'],
+                           v_call=db['v_call'],
+                           references=self.references,
+                           asis_calls=self.asis_calls)
+                imgt_dict = {'sequence_aa_imgt': gap['sequence_imgt'],
+                             'v_germ_aa_start_imgt': gap['v_germ_start_imgt'],
+                             'v_germ_aa_length_imgt': gap['v_germ_length_imgt']}
+            except KeyError as e:
+                imgt_dict = {'sequence_aa_imgt': None,
+                             'v_germ_aa_start_imgt': None,
+                             'v_germ_aa_length_imgt': None}
+                printWarning(e)
+            db.update(imgt_dict)
+            del db['sequence_aa_trim']
+
+        # Add FWR and CDR regions
+        rd = RegionDefinition(junction_length=db.get('junction_length', None),
+                              amino_acid=True, definition=self.regions)
+        regions = rd.getRegions(db.get('sequence_aa_imgt', None))
+        regions = {'fwr1_aa_imgt': regions['fwr1_imgt'],
+                   'fwr2_aa_imgt': regions['fwr2_imgt'],
+                   'fwr3_aa_imgt': regions['fwr3_imgt'],
+                   'cdr1_aa_imgt': regions['cdr1_imgt'],
+                   'cdr2_aa_imgt': regions['cdr2_imgt']}
+        db.update(regions)
+
+        return db
+
+
 class IHMMuneReader:
     """
     An iterator to read and parse iHMMune-Align output files.
@@ -1759,13 +1881,13 @@ class IHMMuneReader:
           dict : database entries for gene calls.
         """
         # Extract allele calls
-        v_call = parseAllele(record['V_CALL'], v_allele_regex, action='list')
-        d_call = parseAllele(record['D_CALL'], d_allele_regex, action='list')
-        j_call = parseAllele(record['J_CALL'], j_allele_regex, action='list')
+        v_call = getVAllele(record['V_CALL'], action='list')
+        d_call = getDAllele(record['D_CALL'], action='list')
+        j_call = getJAllele(record['J_CALL'], action='list')
 
         # Locus
-        locus_list = [parseAllele(record['V_CALL'], locus_regex, action='first'),
-                      parseAllele(record['J_CALL'], locus_regex, action='first')]
+        locus_list = [getLocus(record['V_CALL'], action='first'),
+                      getLocus(record['J_CALL'], action='first')]
         locus = set(filter(None, locus_list))
 
         # Build return object
@@ -1992,14 +2114,17 @@ class IHMMuneReader:
                                       j_germ_start=db['j_germ_start'],
                                       j_germ_length=db['j_germ_length'],
                                       j_call=db['j_call'],
-                                      references=self.references)
+                                      references=self.references,
+                                      regions='default')
             db.update(junc_dict)
 
         # Overall alignment score
         db.update(IHMMuneReader._parseScores(record))
 
         # FWR and CDR regions
-        db.update(getRegions(db.get('sequence_imgt', None), db.get('junction_length', None)))
+        rd = RegionDefinition(junction_length=db.get('junction_length', None),
+                              amino_acid=False, definition='default')
+        db.update(rd.getRegions(db.get('sequence_imgt', None)))
         
         return db
 
@@ -2066,7 +2191,7 @@ def readGermlines(references, asis=False):
         with open(file_name, 'rU') as file_handle:
             germlines = SeqIO.parse(file_handle, 'fasta')
             for g in germlines:
-                germ_key = parseAllele(g.description, allele_regex, 'first') if not asis else g.id
+                germ_key = getAllele(g.description, 'first') if not asis else g.id
                 repo_dict[germ_key] = str(g.seq).upper()
 
     return repo_dict
@@ -2200,10 +2325,18 @@ def getFormatOperators(format):
         reader = ChangeoReader
         writer = ChangeoWriter
         schema = ChangeoSchema
+    elif format == 'changeo-aa':
+        reader = ChangeoReader
+        writer = ChangeoWriter
+        schema = ChangeoSchemaAA
     elif format == 'airr':
         reader = AIRRReader
         writer = AIRRWriter
         schema = AIRRSchema
+    elif format == 'airr-aa':
+        reader = AIRRReader
+        writer = AIRRWriter
+        schema = AIRRSchemaAA
     else:
         raise ValueError
 
@@ -2297,7 +2430,7 @@ def getOutputHandle(file, out_label=None, out_dir=None, out_name=None, out_type=
         printError('File %s cannot be opened.' % out_file)
 
 
-def checkFields(attributes, header, schema=ChangeoSchema):
+def checkFields(attributes, header, schema=AIRRSchema):
     """
     Checks that a file header contains a required set of Receptor attributes
 
